/***************************************************************************
 *   Copyright (C) 2007 by Saritha Kalyanam                                *
 *   sari.kaly@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <string.h>

#include "azureus_vivaldi.h"

static int azureus_vivaldi_v1_decode(struct pkt *pkt, 
                                        struct azureus_vivaldi_pos *pos);

int
azureus_vivaldi_decode(struct pkt *pkt, int type, 
                            struct azureus_vivaldi_pos *pos)
{
    int ret;

    ASSERT(pkt && pos);

    bzero(pos, sizeof(struct azureus_vivaldi_pos));

    ret = pkt_mark(pkt, 512);
    if (ret != SUCCESS) {
        return ret;
    }

    switch (type) {
        case POSITION_TYPE_VIVALDI_V1:
            ret = azureus_vivaldi_v1_decode(pkt, pos);
            if (ret != SUCCESS) {
                return ret;
            }
            break;

        case POSITION_TYPE_VIVALDI_V2:
            break;

        default:
            pkt_reset(pkt);
            return FAILURE;
    }

    return SUCCESS;
}

static int
azureus_vivaldi_v1_decode(struct pkt *pkt, struct azureus_vivaldi_pos *pos)
{
    float f[4];
    int i;
    int ret;

    ASSERT(pkt && pos);

    for (i = 0; i < 4; i++) {
        ret = pkt_read_float(pkt, &f[i]);
        if (ret != SUCCESS) {
            return ret;
        }
    }

    pos->type = POSITION_TYPE_VIVALDI_V1;
    pos->v.v1.x = f[0];
    pos->v.v1.y = f[1];
    pos->v.v1.h = f[2];
    pos->v.v1.error = f[3];

    return SUCCESS;
}
