/***************************************************************************
 *   Copyright (C) 2007 by Saritha Kalyanam   				   *
 *   sari.kaly@gmail.com   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __STUN_H__
#define __STUN_H__

#include <netinet/in.h>

#define STUN_PORT       3478

#define MAX_STUN_TIMEOUT    10

typedef struct { unsigned int id[4]; } __attribute__((packed)) stun_trans_id;

struct stun_header {
    unsigned short msgtype;
    unsigned short msglen;
    stun_trans_id  id;
    unsigned char ies[0];
} __attribute__((packed));

struct stun_attr {
    unsigned short attr;
    unsigned short len;
    unsigned char value[0];
} __attribute__((packed));

struct stun_addr {
    unsigned char unused;
    unsigned char family;
    unsigned short port;
    unsigned int addr;
} __attribute__((packed));

#define STUN_IGNORE     (0)
#define STUN_ACCEPT     (1)

#define STUN_BINDREQ    0x0001
#define STUN_BINDRESP   0x0101
#define STUN_BINDERR    0x0111
#define STUN_SECREQ     0x0002
#define STUN_SECRESP    0x0102
#define STUN_SECERR     0x0112

#define STUN_MAPPED_ADDRESS     0x0001
#define STUN_RESPONSE_ADDRESS   0x0002
#define STUN_CHANGE_REQUEST     0x0003
#define STUN_SOURCE_ADDRESS     0x0004
#define STUN_CHANGED_ADDRESS    0x0005
#define STUN_USERNAME           0x0006
#define STUN_PASSWORD           0x0007
#define STUN_MESSAGE_INTEGRITY  0x0008
#define STUN_ERROR_CODE         0x0009
#define STUN_UNKNOWN_ATTRIBUTES 0x000a
#define STUN_REFLECTED_FROM     0x000b


extern const char * stun_server_list[];

int stun_get_ext_ip_addr(struct sockaddr_in *local_addr4, 
                            struct sockaddr_in *ext_addr4);

void stun_request(int sock, struct sockaddr_in *suggestion,
        const char *username);
int stun_get_mapped_addr(u_int8_t *data, int len, 
        struct sockaddr_in *addr);

#endif /* __STUN_H__ */
